"""
TODO doc
"""

import sys
import os
import random
import hashlib

import pprint
import timeit
import functools

import networkx as nx
import matplotlib.pyplot as plt

from rdflib import plugin, Graph, Literal, URIRef
from rdflib import RDF, FOAF, XSD, DC
from rdflib import namespace
from rdflib.namespace import Namespace, ClosedNamespace, URIPattern
from rdflib.store import Store
from rdflib.extras.external_graph_libs import rdflib_to_networkx_multidigraph
from rdflib.util import guess_format

def queryAll(g):
    qres = g.query("""
        SELECT ?s ?p ?o
        WHERE {
            ?s ?p ?o
        }
        LIMIT 400
    """)
    print(len(qres))

    qres = g.query(
    """
       SELECT DISTINCT ?aname ?bname
       WHERE {
          ?a foaf:knows ?b .
          ?a foaf:name ?aname .
          ?b foaf:name ?bname .
       }
    """, initNs={ 'foaf': FOAF }, result="json")

    print("number of tiplets found: {}".format(len(qres)))
    for row in qres: pass #print("%s knows %s" % row)

def addTipletsWithAlias(g):
    namespace_ejerico = Namespace("http://ejerico.org#")
    g.bind("ejerico", namespace_ejerico)

    u = URIPattern("http://ejerico.org#{}")
    for a in namespace.__dict__:
        if isinstance(namespace.__dict__[a],Namespace) or isinstance(namespace.__dict__[a],ClosedNamespace):
            g.bind(a.lower(), namespace.__dict__[a])

    for i in range(0,1):
        s_uid0 = "{}.{}".format(os.urandom(10),os.urandom(10))
        h_uid0 = hashlib.sha1(s_uid0.encode("utf-8")).hexdigest()

        person = URIRef(u.format(h_uid0))
        g.add((person, RDF.type, FOAF.Person))

        s_uid1 = "{}.{}".format(os.urandom(10),os.urandom(10))
        h_uid1 = hashlib.sha1(s_uid1.encode("utf-8")).hexdigest()
        alias = URIRef(u.format(h_uid1))
        for i in range(5):
            g.add((person, DC.identifer, alias))

        g.add((person, FOAF.mbox, URIRef("mailto:{}@ejerico.org".format(h_uid0))))


    # qres = g.query(
    # """
    #    SELECT ?a 
    #    WHERE {
    #       ?a a foaf.person .
    #    }
    # """, initNs={ 'rdf': RDF, 'foaf': FOAF })

    # print("number of tiplets found: {}".format(len(qres)))
    # for row in qres: pass #print("%s knows %s" % row) 


def addTiplets(g):
    donnaID_1 = URIRef("http://karakola/person/dona_1")
    donnaID_2 = URIRef("http://karakola/person/dona_2")

    for i in range(0,10):
        findme = URIRef("http://karakola/person/find_me".format(random.uniform(1,1000)))
        g.add((findme, RDF.type, FOAF.Person))
        g.add((findme, FOAF.knows, donnaID_1))
        g.add((findme, FOAF.knows, donnaID_2))
        g.add((findme, FOAF.mbox, URIRef("mailto:find.me@example.org")))

    donna = URIRef("http://karakola/person/dona_{}".format(random.uniform(1,1000)))
    g.add((donna, RDF.type, FOAF.Person))
    g.add((donna, FOAF.nick, Literal("donna_{}".format(random.uniform(1,1000)), lang="ed")))
    g.add((donna, FOAF.name, Literal("Donna Fales {}".format(random.uniform(1,1000)))))
    g.add((donna, FOAF.name, Literal("Donna Fales {}".format(random.uniform(1,1000)))))
    g.add((donna, FOAF.knows, donnaID_1))
    g.add((donna, FOAF.knows, donnaID_2))
    g.add((donna, FOAF.mbox, URIRef("mailto:donna.{}@example.org".format(random.uniform(1,1000)))))

    ed = URIRef("http://example.org/edward_{}".format(random.uniform(1,1000)))
    g.add((ed, RDF.type, FOAF.Person))
    g.add((ed, FOAF.nick, Literal("ed_{}".format(random.uniform(1,1000)), datatype=XSD.string)))
    g.add((ed, FOAF.name, Literal("Edward ScissorHands_{}".format(random.uniform(1,1000)))))
    g.add((ed, FOAF.mbox, URIRef("mailto:e.scissorhands.{}@example.org".format(random.uniform(1,1000)))))
    g.add((ed, FOAF.knows, donna))

def main():
    my_ident = URIRef("ejerico")

    datapath = "{}{}data{}".format(os.path.dirname(os.path.realpath(__file__)), os.sep, os.sep)
    os.makedirs(datapath, exist_ok=True)
    my_db_con = "sqlite:///{}rdflib_ejerico_store.db".format(datapath)
    my_db_con = "sqlite:////home/malcalde/.ejerico/harvester.db".format(datapath)

    """ my_db_con = "postgres://{}:{}@{}:{}/{}".format(
        os.environ.get("EJERICO_DB_USERNAME"),
        os.environ.get("EJERICO_DB_PASSWORD"),
        os.environ.get("EJERICO_DB_HOST"),
        os.environ.get("EJERICO_DB_PORT"),
        os.environ.get("EJERICO_DB_SCHEMA")) """

    my_db_con = "postgres://{}:{}@{}:{}/{}".format(
        "ejerico",
        "ejerico",
        "postgres.test.socib.es",
        "5433",
        "js3socib") 

    my_uri = Literal(my_db_con) 

    #store = plugin.get("SQLAlchemy", Store)(identifier=my_ident)
    store = plugin.get("EJERICOStore", Store)(identifier=my_ident)
    
    #g = Graph(store="SQLAlchemy", identifier=my_ident)
    g = Graph(store="EJERICOStore", identifier=my_ident)
    a = g.open(my_uri, create=True)

    print(guess_format("/home/xxx/thing.json"))
    print(guess_format("/home/xxx/thing.xml"))
    print(guess_format("/home/xxx/thing.ttl"))
    print(guess_format("/home/xxx/thing.turtle"))

    #t_update = timeit.Timer(functools.partial(addTipletsWithAlias, g)) 
    #for _ in range(0,2): print(t_update.timeit(2))

    #t_update = timeit.Timer(functools.partial(addTiplets, g)) 
    #for i in range(0,10): print(t_update.timeit(2))

    t_query = timeit.Timer(functools.partial(queryAll, g)) 
    print(t_query.timeit(1))

    a = g.serialize(format="turtle").decode("utf-8")   
    pprint.pprint(40*'#')
    pprint.pprint(a)


    #b = g.serialize(format='json-ld').decode('utf-8')
    #pprint.pprint(40*'#')
    #pprint.pprint(b)

    #g.destroy(my_uri)
    try:
        g.close()
    except:
        pass

if __name__ == "__main__":
    main()
